import django.contrib.auth
from django.utils import timezone
from dinero_auth.crypto import  Crypt
from dinero_auth.models import DineroToken
import json
from django.contrib.auth.models import User
from rest_framework.authentication import  get_authorization_header

class AuthToken:
    
    def __init__(self,request) -> None:
        
        self.__request = request
        
    def create_token(self,data):
        strToEnc = json.dumps(data)
        crypt = Crypt()
        token = crypt.encrypt(strToEnc)
        return token
        
    def get_user_ip(self) -> str:
        
        x_forwarded_for = self.__request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            ip = x_forwarded_for.split(',')
        else:
            ip = self.__request.META.get('REMOTE_ADDR')
        
        return ip
    
    def get_user_browser_info(self) -> str:
        
        return self.__request.META['HTTP_USER_AGENT']
        
    def create(self,id) -> str:
        
        ip = self.get_user_ip()
        browser = self.get_user_browser_info()
        user = User.objects.get(id = id)
        
        tkn_obj = DineroToken.objects.create(user = user ,browser = browser ,ip = ip ,)
        
        data =  {
            "id" : tkn_obj.id,
            "user" : user.id,
            "ip" : ip,
            "browser" : browser
        }
        
        token = self.create_token(data)
        tkn_obj.token = token
        tkn_obj.save()
        
        return token
    
    def get_token_information(self,token):
        plainText = Crypt().decrypt(token)
        data = json.loads(plainText)
        return data["id"]
    
    def logout(self):
        
        auth = get_authorization_header(self.__request).split()
        
        tkn_id = self.get_token_information(auth[1].decode())
        
        if DineroToken.objects.filter(id = tkn_id).exists():
            DineroToken.objects.filter(id = tkn_id).delete()


from django.conf import settings
import django.db
from rest_framework import exceptions
from rest_framework.authentication import (
    BaseAuthentication, get_authorization_header,
)
from django.utils import timezone
from dinero_auth.crypto import  Crypt
from dinero_auth.models import DineroToken
from django.contrib.auth.models import User
import json


class DineroTokenAuthentication(BaseAuthentication):
    
    def authenticate(self,request):
        auth = get_authorization_header(request).split()
        prefix = settings.AUTH_HEADER_PREFIX.encode()

        if not auth:
            return None
        
        if auth[0].lower() != prefix.lower():
            # Authorization header is possibly for another backends
            return None
        
        if len(auth) == 1:
            msg = 'Invalid token header. No credentials provided.'
            raise exceptions.AuthenticationFailed(msg)
        
        elif len(auth) > 2:
            msg = 'Invalid token header. Token string should not contain spaces.'
            raise exceptions.AuthenticationFailed(msg)
        user , auth_token = self.authenticate_credentials(request,auth[1].decode())
        return(user, auth_token)

    def authenticate_credentials(self,request, token):

        plainText = Crypt().decrypt(token)
        data = json.loads(plainText)
        valid,id = self.validate_data(request,data)
        
        if valid:
            auth_token = DineroToken.objects.get(id = id)
            current_access_expiry = auth_token.access_expiry
            
            if self.validate_user(auth_token.user):
                if current_access_expiry >= timezone.now():
                    self.refresh_token(auth_token)
                    
                else:
                    msg = 'The authentication token provided is expired. Please Relogin!'
                    raise exceptions.AuthenticationFailed(msg)
                return auth_token.user,auth_token.token
        else:
            msg = 'The authentication token provided is invalid or corrupt.'
            raise exceptions.AuthenticationFailed(msg)
        
    def refresh_token(self,auth_token):
        return True
        
    
    def validate_user(self,user):
        if not user.is_active:
                    raise exceptions.AuthenticationFailed('User inactive or deleted.')
        return True
            
    def validate_data(self,request,data):
        
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            crnt_ip = x_forwarded_for.split(',')[0]
        else:
            crnt_ip = request.META.get('REMOTE_ADDR')
        
        crnt_browser = request.META['HTTP_USER_AGENT']
        if DineroToken.objects.filter(id = data["id"]).exists():
            tkn = DineroToken.objects.get(id = data["id"])
            
            v_browser = crnt_browser == data["browser"] == tkn.browser
            v_ip = crnt_ip == data["ip"] == tkn.ip
            v_user = data["user"] == tkn.user.id
        
            if any(not value for value in [v_ip,v_browser,v_user]):
                msg = 'The authentication token provided is invalid or corrupt.'
                raise exceptions.AuthenticationFailed(msg)

            return True,data["id"]
        return False,None


from django.db import models
from django.contrib.auth.models import User
from django.utils import timezone
from django.conf import settings

class DineroTokenManager(models.Manager):
    
    def create(self,user ,browser , ip):
        access_TTL = settings.ACCESS_TOKEN_TTL

        if access_TTL is not None:
            access_expiry = timezone.now() + access_TTL

        instance = super(DineroTokenManager, self).create(
            access_expiry = access_expiry,
            user = user,
            browser = browser,
            ip = ip
            )
        return instance


class DineroToken(models.Model):
    
    objects = DineroTokenManager()
    
    token = models.CharField( max_length=500 , null=True, blank=True )

    user = models.ForeignKey(User, null=False, blank=False, related_name='auth_token', on_delete=models.CASCADE)
    
    created = models.DateTimeField(auto_now_add=True)
    
    access_expiry = models.DateTimeField(null=True, blank=True)
    
    ip = models.CharField( max_length=500 )
    
    browser = models.CharField( max_length=500 )

    def __str__(self):
        return '%s : %s' % (self.id, self.user)
    
    class Meta:
        db_table = 'dn_token'




from django.conf import settings
from rest_framework import exceptions

import json 
from Crypto.Cipher import AES
from base64 import b64encode, b64decode

class Crypt:
    def __init__(self):
        salt = settings.SALT
        self.salt = salt.encode()
        self.enc_dec_method = 'utf-8'
        self.token_key = settings.TOKEN_KEY

    def encrypt(self, str_to_enc):
        aes_obj = AES.new(self.token_key.encode(), AES.MODE_CFB, self.salt)
        hx_enc = aes_obj.encrypt(str_to_enc.encode())
        mret = b64encode(hx_enc).decode(self.enc_dec_method)
        return mret

    def decrypt(self, enc_str):
        try:
            aes_obj = AES.new(self.token_key.encode(), AES.MODE_CFB, self.salt)
            str_tmp = b64decode(enc_str.encode(self.enc_dec_method))
            str_dec = aes_obj.decrypt(str_tmp)
            mret = str_dec.decode(self.enc_dec_method)
        except:
            msg = 'Invalid token Please relogin.'
            raise exceptions.AuthenticationFailed(msg)
        return mret
    